/*
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Bonneau and Obeo - initial API and implementation
 */

package fr.obeo.acceleo.gen.debug.ui.model;

import org.eclipse.debug.core.DebugException;
import org.eclipse.debug.core.model.IBreakpoint;
import org.eclipse.debug.core.model.IStackFrame;
import org.eclipse.debug.core.model.IThread;

import fr.obeo.acceleo.gen.template.debug.ITemplateDebugger;
import fr.obeo.acceleo.gen.template.debug.StackInfo;

/**
 * A Acceleo VM thread. A Acceleo VM is single threaded.
 */
public class AcceleoThread extends AcceleoDebugElement implements IThread {

	/**
	 * Breakpoints this thread is suspended at or <code>null</code> if none.
	 */
	private IBreakpoint[] fBreakpoints;

	/**
	 * Whether this thread is stepping
	 */
	private boolean fStepping = false;

	private boolean suspended = false;

	/**
	 * Constructs a new thread for the given target
	 * 
	 * @param target
	 *            VM
	 */
	public AcceleoThread(AcceleoDebugTarget target) {
		super(target);
	}

	/* (non-Javadoc) */
	public IStackFrame[] getStackFrames() throws DebugException {
		AcceleoDebugTarget debugTarget = (AcceleoDebugTarget) getDebugTarget();
		ITemplateDebugger debugger = debugTarget.getDebugger();
		StackInfo[] stack = debugger.getStack();
		IStackFrame[] ret = new IStackFrame[stack.length];
		for (int i = 0; i < ret.length; ++i) {
			ret[i] = new AcceleoStackFrame(this, stack[stack.length - i - 1]);
		}
		return ret;
	}

	/* (non-Javadoc) */
	public boolean hasStackFrames() throws DebugException {
		return isSuspended();
	}

	/* (non-Javadoc) */
	public int getPriority() throws DebugException {
		return 0;
	}

	/* (non-Javadoc) */
	public IStackFrame getTopStackFrame() throws DebugException {
		IStackFrame[] frames = getStackFrames();
		if (frames.length > 0) {
			return frames[0];
		}
		return null;
	}

	/* (non-Javadoc) */
	public String getName() throws DebugException {
		return "Thread[1]"; //$NON-NLS-1$
	}

	/* (non-Javadoc) */
	public IBreakpoint[] getBreakpoints() {
		if (fBreakpoints == null) {
			return new IBreakpoint[0];
		}
		return fBreakpoints;
	}

	/**
	 * Sets the breakpoints this thread is suspended at, or <code>null</code>
	 * if none.
	 * 
	 * @param breakpoints
	 *            the breakpoints this thread is suspended at, or
	 *            <code>null</code> if none
	 */
	protected void setBreakpoints(IBreakpoint[] breakpoints) {
		fBreakpoints = breakpoints;
	}

	/* (non-Javadoc) */
	public boolean canResume() {
		return isSuspended();
	}

	/* (non-Javadoc) */
	public boolean canSuspend() {
		return !isSuspended();
	}

	/* (non-Javadoc) */
	public boolean isSuspended() {
		return suspended;
	}

	/* (non-Javadoc) */
	public void resume() throws DebugException {
		getDebugTarget().resume();
	}

	/* (non-Javadoc) */
	public void suspend() throws DebugException {
		getDebugTarget().suspend();
	}

	/* (non-Javadoc) */
	public boolean canStepInto() {
		return isSuspended();
	}

	/* (non-Javadoc) */
	public boolean canStepOver() {
		return isSuspended();
	}

	/* (non-Javadoc) */
	public boolean canStepReturn() {
		return isSuspended();
	}

	/* (non-Javadoc) */
	public boolean isStepping() {
		return fStepping;
	}

	/* (non-Javadoc) */
	public void stepInto() throws DebugException {
		((AcceleoDebugTarget) getDebugTarget()).stepInto();
	}

	/* (non-Javadoc) */
	public void stepOver() throws DebugException {
		((AcceleoDebugTarget) getDebugTarget()).stepOver();
	}

	/* (non-Javadoc) */
	public void stepReturn() throws DebugException {
		((AcceleoDebugTarget) getDebugTarget()).stepReturn();
	}

	/* (non-Javadoc) */
	public boolean canTerminate() {
		// return !isTerminated();
		return false;
	}

	/* (non-Javadoc) */
	public boolean isTerminated() {
		return getDebugTarget().isTerminated();
	}

	/* (non-Javadoc) */
	public void terminate() throws DebugException {
		getDebugTarget().terminate();
	}

	/**
	 * Sets whether this thread is stepping.
	 * 
	 * @param stepping
	 */
	protected void setStepping(boolean stepping) {
		fStepping = stepping;
	}

	/**
	 * Sets whether this thread is suspended.
	 * 
	 * @param suspended
	 */
	protected void setSuspended(boolean suspended) {
		this.suspended = suspended;
	}
}
